about summary refs log tree commit diff
path: root/src/math/tanl.c
diff options
context:
space:
mode:
authorSzabolcs Nagy <nsz@port70.net>2013-05-18 14:40:22 +0000
committerSzabolcs Nagy <nsz@port70.net>2013-05-18 14:40:22 +0000
commitbfda37935867f9bf271d6074db0accf05c63ad10 (patch)
tree92d7fffcc6fb7fd9caad6f3bde14dcd768d32b66 /src/math/tanl.c
parent1d5ba3bb5a3f55e10db05219638cfcd967d65417 (diff)
downloadmusl-bfda37935867f9bf271d6074db0accf05c63ad10.tar.gz
musl-bfda37935867f9bf271d6074db0accf05c63ad10.tar.xz
musl-bfda37935867f9bf271d6074db0accf05c63ad10.zip
math: sin cos cleanup
* use unsigned arithmetics
* use unsigned to store arg reduction quotient (so n&3 is understood)
* remove z=0.0 variables, use literal 0
* raise underflow and inexact exceptions properly when x is small
* fix spurious underflow in tanl
Diffstat (limited to 'src/math/tanl.c')
-rw-r--r--src/math/tanl.c11
1 files changed, 6 insertions, 5 deletions
diff --git a/src/math/tanl.c b/src/math/tanl.c
index 3b51e011..546c7a02 100644
--- a/src/math/tanl.c
+++ b/src/math/tanl.c
@@ -53,11 +53,12 @@ long double tanl(long double x)
 
 	/* |x| < (double)pi/4 */
 	if (z.e < M_PI_4) {
-		/* x = +-0 or x is subnormal */
-		if (z.bits.exp == 0)
-			/* inexact and underflow if x!=0 */
-			return x + x*0x1p-120f;
-		/* can raise spurious underflow */
+		/* |x| < 0x1p-64 */
+		if (z.bits.exp < 0x3fff - 64) {
+			/* raise inexact if x!=0 and underflow if subnormal */
+			FORCE_EVAL(z.bits.exp == 0 ? x/0x1p120f : x+0x1p120f);
+			return x;
+		}
 		return __tanl(x, 0, 0);
 	}